
// Simple multithread-safe sound queue. Write samples into queue then read them
// from a sound callback in a different thread.

// Copyright (C) 2005 Shay Green. MIT license.

#ifndef SOUND_QUEUE_H
#define SOUND_QUEUE_H

class Sound_Queue {
public:
	// Sample type
	typedef short sample_t;
	
	// Queue can hold at most 'max_size' samples
	Sound_Queue( long max_size );
	~Sound_Queue();
	
	// Remove any queued samples. Not thread-safe.
	void clear();
	
	// Write 'count' samples to queue, waiting until enough space is available.
	void write( const sample_t*, long count );
	
	// Number of samples available for reading
	long avail() const;
	
	// Read at most 'count' samples from queue and return number
	// actually read
	long read( sample_t*, long count );

private:
	sample_t* volatile buf;
	volatile long write_pos;
	volatile long read_pos;
	volatile long max_size;
	
	// Optional direct buffer access so samples can be written directly into queue
	
	// Number of samples available in buffer
	long buffer_avail() const;
	
	// Pointer to buffer where at most 'buffer_avail()' new samples
	// can be written
	sample_t* buffer() const;
	
	// Add 'count' newly written samples to queue
	void write( long count );
};

// end of public interface

	inline Sound_Queue::sample_t* Sound_Queue::buffer() const {
		return buf + write_pos;
	}

#endif

