#include "../emu/h/types.h"
#include "h/cpu.h"
#include "h/nesGeneral.h"
#include "h/ppu.h"
#include "h/interruptions.h"

namespace Reminesce
{

u8 readPPU();
void writePPU(u8 val);

void write2000(u8 val)
{
	if ( ppu.vblankBit && (!(ppu.r2000 & 0x80)) && (val & 0x80))
		NMI();

	ppu.r2000 = val;
	ppu.addrInc = (val & 4) ? 32 : 1;
	ppu.temp = (ppu.temp & 0xF3FF) | ((val & 3)<<10);
}

void write2001(u8 val)
{
	ppu.r2001 = val;
}

u8 read2002()
{
	ppu.bus &= 0x1F;
	ppu.bus |= ppu.vblankBit | ppu.sprite0Hit | ppu.spriteOver;
	ppu.vblankBit = ppu.secondWrite = 0;
	return ppu.bus;
}

void write2003(u8 val)
{
	ppu.sprLoc = val;
}

u8 read2004()
{
	ppu.bus = sprRam[ppu.sprLoc];
	if ((ppu.sprLoc & 3) == 2)
		ppu.bus &= 0xE3;

	return ppu.bus;
}

void write2004(u8 val)
{
	sprRam[ppu.sprLoc++] = val;
}

void write2005(u8 val)
{
	if (ppu.secondWrite)
	{
		ppu.temp &= 0xC1F;
		ppu.temp |= ( ((val & 0xF8)<<(5-3)) |((val & 7)<<12) );
		ppu.secondWrite = 0;
	}
	else	//Write 1
	{
		ppu.temp  &= 0xFFE0;
		ppu.xLatch = val & 7;
		ppu.temp  |= val>>3;
		ppu.secondWrite = 1;
	}
}

void write2006(u8 val)
{
	if (ppu.secondWrite)
	{
		ppu.temp &= 0xFF00;
		ppu.temp |= val;
		ppu.secondWrite = 0;
		ppu.address = ppu.temp;
	}
	else
	{
		ppu.temp = ((val & 63)<<8) | (u8)ppu.temp; //Zero's the 2 high bits
		ppu.secondWrite = 1;
	}
}

void write2007(u8 val)
{
	writePPU(val);
	ppu.address += ppu.addrInc;
}

u8 read2007()
{
	ppu.bus = ppu.pipe;
	readPPU();
	ppu.address += ppu.addrInc;
	return ppu.bus;
}

void writePPU(u8 val)
{
	if ((ppu.address & 0x3FFF) < 0x2000)
	{
			chrRam[ppu.address & 0x1FFF] = val;
	}
	else if((ppu.address & 0x3FFF) >= 0x3F00)
	{
		if (!(ppu.address & 3))
			pal[ppu.address & 0x0F] = val & 0x3F;
		else
			pal[ppu.address & 0x1F] = val & 0x3F;
	}
	else	//ppu.address is < 0x3F00
		pntBank[(ppu.address>>10) & 3][ppu.address & 0x3FF] = val;
}

u8 readPPU()
{
	if ((ppu.address & 0x3FFF)<0x2000)
		ppu.pipe = pChrRead[(ppu.address>>10) & 0x7][ppu.address & 0x3ff];
	else if((ppu.address & 0x3FFF) >= 0x3F00)
	{
		register char temp;
		temp = (ppu.r2001 & 1) ? 0x30 : 0x3F;
		if (!(ppu.address & 3))
			temp = pal[ppu.address & 0x0F] & temp;
		else
			temp = pal[ppu.address & 0x1F] & temp;
		ppu.pipe = ppu.bus =  temp;
	}
	else	//ppu.address is <0x3F00
		ppu.pipe = pntBank[(ppu.address>>10) & 3][ppu.address & 0x3FF];

	return ppu.bus;
}

}
