#include "../emu/h/interface.h"
#include "../emu/h/mapper.h"
#include "../emu/h/emuPalette.h"
#include "h/ppu.h"
#include "h/cpu.h"
#include "h/nesGeneral.h"
#include "h/interruptions.h"

#include "blargg/Nes_Apu.h"
#include "h/apu.h"

namespace Reminesce
{

void runNesPAL();

extern CPMapperInfo mapInfo;

inline void RefreshLine()
{
	if (!(ppu.r2001 & 0x18))
		return;

	ppu.address += 1<<12;			//increase fine y
	if (ppu.address & 0x8000)		//if fine y is done reset it and use next 32 tiles
	{
		if ((ppu.address & 0x3E0)==0x3A0)
			ppu.address  = (ppu.address & 0x800) ^ 0x800;	//change Y NT
		else if ((ppu.address & 0x3E0)==0x3E0)
			ppu.address  = ppu.address & 0x800;
		else
			ppu.address  = (ppu.address + 32) & 0x7BE0;		//keep Y NT, increase Tile Y, clear tile X
	}
	else
		ppu.address &= ~0x41f;
	ppu.address |= ppu.temp & 0x41F;	
}

void runNes()
{
	nesMode = nesMode & (-1 ^ resetLoop);

	if (!imageLoaded)
	{
		brunNes  = 0;
		nesMode &= (-1 ^ inRunCpu);
		return;
	}

	if ((nesMode & regionModeAnd) == palMode)
	{
		runNesPAL();
		return;
	}
	ui pitch, n;
	void *surface;

	if (nesMode & inRunCpu)
		return;

	nesMode |= inRunCpu;
	FrameDef();

	BuildKeys();					//sometimes the first call gets you junk
	BuildKeys();

	cpuCycles -= cpuTargetCycles;
	cpuTargetCycles = 0;

	while (brunNes)
	{
		ppu.vblankBit = ppu.sprite0Hit = ppu.spriteOver = 0;	//Clear 2002 at beginning of each frame

		StartFrame(&pitch, &surface);
////////////////////////////////////////////////////////////////////////////////////

		if (ppu.r2001 & 0x18)
			ppu.address = ppu.temp;
#ifndef dummySkip
		runCpu(86);
		if (mapInfo->HBlank)
			mapInfo->HBlank(0, ppu.r2001);

		apu.run_until(cpuCycles);

		runCpu(28);
		RefreshLine();
		for (n=0; n<80; n++)
		{
			runCpu(86);
			RenderTheLine(n*3 + 0, surface, pitch);
			apu.run_until(cpuCycles);

			if (mapInfo->HBlank)
				mapInfo->HBlank(n*3 + 1, ppu.r2001);
			runCpu(28);
			RefreshLine();

			runCpu(85);
			RenderTheLine(n*3 + 1, surface, pitch);
			apu.run_until(cpuCycles);

			if (mapInfo->HBlank)
				mapInfo->HBlank(n*3 + 2, ppu.r2001);
			runCpu(28);
			apu.run_until(cpuCycles);
			RefreshLine();

			runCpu(86);
			RenderTheLine(n*3 + 2, surface, pitch);
			apu.run_until(cpuCycles);

			if (mapInfo->HBlank)
				mapInfo->HBlank(n*3 + 3, ppu.r2001);
			runCpu(28);
			RefreshLine();
		}

		runCpu(114-7);					// stop a few cycles early for those pain in the ass games that set NMI AND reads 2002 for vblank
		apu.run_until(cpuCycles);
		ppu.vblankBit = 0x80;
		runCpu(7);
		apu.run_until(cpuCycles);

		if (ppu.r2000 & 0x80)			//Check for vblank, then jump
			NMI();

		if (mapInfo->HBlank)
			mapInfo->HBlank(240, ppu.r2001);

		for (n=0; n<6; n++)
		{
			runCpu(113);
			if (mapInfo->HBlank)
				mapInfo->HBlank(241 + n*3, ppu.r2001);

			apu.run_until(cpuCycles);
			runCpu(114);

			if (mapInfo->HBlank)
				mapInfo->HBlank(242 + n*3, ppu.r2001);

			apu.run_until(cpuCycles);
			runCpu(114);
			if (mapInfo->HBlank)
				mapInfo->HBlank(243 + n*3, ppu.r2001);

			apu.run_until(cpuCycles);
		}


		if (apu.earliest_irq() <= cpuCycles)
			IRQ();

		runCpu(113);
		if (mapInfo->HBlank)
			mapInfo->HBlank(259, ppu.r2001);
		
		runCpu(114);
		if (mapInfo->HBlank)
			mapInfo->HBlank(260, ppu.r2001);

		apu.end_frame(cpuCycles);
		blipBuf.end_frame(cpuCycles);
		playSound();

		cpuCycles -= cpuTargetCycles;		//do it here so state allways has the
		cpuTargetCycles = 0;				//lowest cpuCycles (state uses 3 bits)

		//29781 vs 29780.667
////////////////////////////////////////////////////////////////////////////////////
#endif
		Render();
		EndFrameNTSC();
		BuildKeys();
		RefreshGUI();
	}
	nesMode &= (-1 ^ inRunCpu);
}



void runNesPAL()
{
	ui pitch, n, i;
	void *surface;

	if (nesMode & inRunCpu)
		return;

	nesMode |= inRunCpu;

	FrameDef();

	BuildKeys();					//sometimes the first call gets you junk
	BuildKeys();

	if (!imageLoaded)
	{
		brunNes=0;
		nesMode &= (-1 ^ inRunCpu);
		return;
	}

	cpuCycles -= cpuTargetCycles;
	cpuTargetCycles = 0;
	memloc.si_ = temp.si_ = 0;
	while (brunNes)
	{
		ppu.vblankBit = ppu.sprite0Hit = ppu.spriteOver = 0;	//Clear 2002 at beginning of each frame
		if (ppu.r2001 & 0x18)
			ppu.address = ppu.temp;

		StartFrame(&pitch, &surface);
////////////////////////////////////////////////////////////////////////////////////

		if (ppu.r2001 & 0x18)
			ppu.address = ppu.temp;

		runCpu(80);
		if (mapInfo->HBlank)
			mapInfo->HBlank(0, ppu.r2001);
		
		apu.run_until(cpuCycles);
		runCpu(27);
		RefreshLine();

		for (i=0; i<15; i++)
		{	
			for (n=0; n<4; n++)
			{
				runCpu(80);
				RenderTheLine(i*16 + n*4, surface, pitch);
				apu.run_until(cpuCycles);

				if (mapInfo->HBlank)
					mapInfo->HBlank(i*16 + n*4 + 1, ppu.r2001);
				runCpu(27);
				RefreshLine();


				runCpu(79);
				RenderTheLine(i*16 + n*4 + 1, surface, pitch);
				apu.run_until(cpuCycles);

				if (mapInfo->HBlank)
					mapInfo->HBlank(i*16 + n*4 + 2, ppu.r2001);
				runCpu(27);
				RefreshLine();

											
				runCpu(80);
				RenderTheLine(i*16 + n*4 + 2, surface, pitch);
				apu.run_until(cpuCycles);

				if (mapInfo->HBlank)
					mapInfo->HBlank(i*16 + n*4 + 3, ppu.r2001);
				runCpu(27);
				RefreshLine();


				runCpu(79);
				RenderTheLine(i*16 + n*4 + 3, surface, pitch);
				apu.run_until(cpuCycles);

				if (mapInfo->HBlank)
					mapInfo->HBlank(i*16 + n*4 + 4, ppu.r2001);
				runCpu(27);
				RefreshLine();
			}
			runCpu(1);
		}

		runCpu(107-7);					// stop a a cycles early for those pain in the ass games that set NMI AND reads 2002 for vblank
		apu.run_until(cpuCycles);

		ppu.vblankBit = 0x80;
		runCpu(7);
		apu.run_until(cpuCycles);

		if (ppu.r2000 & 0x80)			//Check for vblank, then jump
			NMI();

		if (mapInfo->HBlank)
			mapInfo->HBlank(241, ppu.r2001);

		for (int i=0; i<35; i++)
		{
			runCpu(106);
			apu.run_until(cpuCycles);

			if (mapInfo->HBlank)
				mapInfo->HBlank(242 + i*2, ppu.r2001);

			runCpu(107);
			if (mapInfo->HBlank)
				mapInfo->HBlank(242 + i*2 + 1, ppu.r2001);
			apu.run_until(cpuCycles);
		}

		if ((apu.earliest_irq()) <= cpuCycles)
			IRQ();

		runCpu(5);
		
		apu.end_frame(cpuCycles);
		blipBuf.end_frame(cpuCycles);
		playSound();

		cpuCycles -= cpuTargetCycles;		//do it here so state allways has the
		cpuTargetCycles = 0;				//lowest cpuCycles (state uses 3 bits)

		//29781 vs 29780.667
////////////////////////////////////////////////////////////////////////////////////
		Render();
		EndFramePAL();
		BuildKeys();
		RefreshGUI();
	}
	nesMode &= (-1 ^ inRunCpu);
}

}
